/*
 * Copyright (C) 2002 Robert Ernst <robert.ernst@linux-solutions.at>
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See COPYING for GPL licensing information.
 *
 */

#ifndef __BOARD_HEADER__
#define __BOARD_HEADER__

#include <qwidget.h>
#include <qpixmap.h>
#include "SpriteCollection.h"

class Board : public QWidget
{
    Q_OBJECT

public:
    Board(QWidget *parent, const char *name = 0, WFlags f = 0);
    ~Board();

public:
    static const int rows = 6;
    static const int cols = 7;
    static const int border = 4;
    enum State {
	PLAYER_MOVE,
	PLAYER_ANIM,
	COMPUTER_MOVE,
	COMPUTER_ANIM,
	HINT,
    };
    enum Chip {
	EMPTY,
	RED,
	YELLOW,
    };

private:
    SpriteCollection *m_sprites;
    QPixmap *m_pixmap;
    bool m_game_running;
    enum State m_state;
    int m_strength;
    bool m_animations;
    bool m_computerBegins;
    int m_chip[rows][cols];
    int m_xoffs;
    int m_yoffs;
    int m_xsize;
    int m_ysize;
    int m_movecount;
    int m_move[rows * cols];

public slots:
    void start(void);
    void back(void);
    void hint(void);
    void updateStrength(int strength);
    void updateAnimations(int animations);
    void updateComputerBegins(int computerBegins);

private:
    bool checkMove(int &col, int &row);
    void animateMove(Chip who, int col, int row);
    void computerMove(void);
    bool findRowOfFour(int col, int row, int dcol, int drow);
    bool moveWon(void);
    bool moveNotPossible(void);
    void gameOver(Chip who);
    void drawBoard(Chip who, int x, int y);
    void backMove(void);
    int computeNextMove(Chip who);

protected:
    void paintEvent(QPaintEvent *event);
    void mousePressEvent(QMouseEvent *event);
};

#endif // __BOARD_HEADER__

